// Geant4 :
#include <G4RunManager.hh>
#include <QGSP_EMV.hh>

// G4LabTemplateFour :
#include <G4LabTemplateFour/Detector.h>
#include <G4LabTemplateFour/Analysis.h>
#include <G4LabTemplateFour/Primary.h>
//#include <G4LabTemplateFour/PhysicsList.h>
#include <G4LabTemplateFour/RunAction.h>
#include <G4LabTemplateFour/EventAction.h>
#include <G4LabTemplateFour/SteppingAction.h>

// AIDA :
#include <AIDA/IAnalysisFactory.h>
#include <AIDA/IHistogram1D.h>
#include <AIDA/IProfile1D.h>
#include <AIDA/ITuple.h>

// std::
#include <iostream>

// Lib :
#include <Lib/Arguments.h>
#include <Lib/smanip.h>

//////////////////////////////////////////////////////////////////////////////
int main(int aArgc,char** aArgv) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Arguments args(aArgc,aArgv);

  int nevt;
  if(!args.find("-nevt",nevt)) nevt = 1000;

  unsigned int dump_rate;
  if(!args.find("-dump_rate",dump_rate)) dump_rate = 100;
  if(!dump_rate) dump_rate = 100;

  // E in GeV :
  double E;
  if(!args.find("-E",E)) E = 1;

  std::string smat;
  args.find("-mat",smat);

  //name,z,a,density.
  G4Material* material = 0;
  double RhoMoliere = 0;
  if(smat=="alu") {
    material = new G4Material("aluminium",13.,26.98*g/mole,2.7*g/cm3);
    RhoMoliere = 44*mm;
  } else if(smat=="copper") {
    material = new G4Material("copper",29.,63.55*g/mole,8.96*g/cm3);
    RhoMoliere = 15.2*mm;
    //X0 = 1.43568 cm
  } else if(smat=="lead") {
    material = new G4Material("lead",82.,207.19*g/mole,11.35*g/cm3);
    RhoMoliere = 16*mm;
    //X0 = 0.56118 cm
  } else {
    std::cout << "G4LabTemplateFour_batch :" 
              << std::endl
              << " unknown material \"" << smat << "\""
              << std::endl
              << " give one with :" 
              << std::endl
              << "   OS> G4LabTemplateFour_batch -mat=[alu,copper,lead]"
              << std::endl
              << " for example :" 
              << std::endl
              << "   UNIX> ./G4LabTemplateFour_batch -mat=alu"
              << std::endl;
    return 1;
  }

  std::string fileName;
  //Lib::smanip::printf(fileName,128,"%s_el_E_%g_GeV_%d_evts",
  //  material->GetName().c_str(),E,nevt);
  Lib::smanip::printf(fileName,128,"%s_el_E_%g_GeV",
    material->GetName().c_str(),E);

  E *= 1*GeV;

  G4RunManager* runManager = new G4RunManager;

  G4LabTemplateFour::Detector* det = 
    new G4LabTemplateFour::Detector(*material);

  runManager->SetUserInitialization(det);
  //runManager->SetUserInitialization(new G4LabTemplateFour::PhysicsList()); 

  G4VUserPhysicsList* pl = new QGSP_EMV();
  double cut = 30*um;
  pl->SetCutValue(cut,"gamma");
  pl->SetCutValue(cut,"e-");
  pl->SetCutValue(cut,"e+");
  //pl->DumpCutValuesTable();

  runManager->SetUserInitialization(pl); // ext QGSP_EMV (EM_7.0)

  runManager->SetUserAction
    (new G4LabTemplateFour::PrimaryGeneratorAction(-1*cm,E));
  runManager->SetUserAction(new G4LabTemplateFour::EventAction(dump_rate));
  runManager->SetUserAction(new G4LabTemplateFour::RunAction());

  //AIDA Analysis factory
  AIDA::IAnalysisFactory* aida = AIDA_createAnalysisFactory();
  if(!aida) {
    std::cout << "G4LabTemplateFour_batch :" 
              << " AIDA not found." 
              << std::endl;
    delete runManager;
    return 1;
  }

  G4LabTemplateFour::Analysis* analysis = 
    new G4LabTemplateFour::Analysis(*aida,*det,fileName);

  runManager->SetUserAction
    (new G4LabTemplateFour::SteppingAction(*analysis,*det,RhoMoliere));

  runManager->Initialize();

  runManager->BeamOn(nevt);

  analysis->f_run->fill(0,std::string("e-"));
  analysis->f_run->fill(1,E);
  analysis->f_run->fill(2,nevt);
  analysis->f_run->addRow();

  std::cout << " material name = " << material->GetName()
            << ", X0 = " << (material->GetRadlen()/cm) 
            << " cm." << std::endl;

  //double S_dE = analysis->f_z_dE->sumBinHeights();
  //S_dE /= GeV;
  //std::cout << " S_dE = " << S_dE << " GeV." << std::endl;
  std::cout << " entries = " << analysis->f_z_dE->entries() << std::endl;
  std::cout << " nevt*E = " << ((nevt*E)/GeV) << " GeV." << std::endl;
  std::cout << " file = " << fileName << std::endl;

  delete runManager; 
  //the upper deletes :
  // Detector, PhysicsList, PrimaryGeneratorAction, EventAction, RunAction

  delete analysis;
  delete aida;

  return 0;
}

