#ifndef OnX_QTk_h
#define OnX_QTk_h 

namespace Slash {namespace Core {class IMessage;}}

namespace OnX {
  class Action;
  class QtUI;
}

#include <qsplitter.h>
#include <qevent.h>
#include <qgl.h>
#include <qdialog.h>

class QCallbackData;

#if QT_VERSION < 0x040000
class QPopupMenu;
class QListViewItem;
#else
#ifdef QT3_SUPPORT
class Q3PopupMenu;
class Q3ListViewItem;
#define QPopupMenu Q3PopupMenu
#else
#error "OnX on Qt4 without -DQT3_SUPPORT not yet supported."
#endif
class QCustomEvent;
#endif

class OnX_QOpenGLArea : public QGLWidget {
  Q_OBJECT
public:
  OnX_QOpenGLArea(QWidget*,const char*);
  virtual ~OnX_QOpenGLArea();
  virtual void initialieGL();
  virtual void resizeGL(int,int);
  virtual void paintGL();
  virtual void mousePressEvent(QMouseEvent*);
  virtual void mouseReleaseEvent(QMouseEvent*);
  virtual void mouseMoveEvent(QMouseEvent*);
signals:
  void paint(QCallbackData&);
  void eventHandler(QCallbackData&);
};

// To handle splitPosition at startup.
class OnX_QSplitter : public QSplitter {
  Q_OBJECT
public:
  OnX_QSplitter(Qt::Orientation,QWidget*,const char*);
  virtual ~OnX_QSplitter();
  void setSplitPosition(int);
  virtual void showEvent(QShowEvent*);
private:
  int fSplitPosition;
};

//WARNING : can't be put in the main panel. The show 
// on the main panel will show it and then we will
// be blocked in the QColorDialog::getColor().

//class OnX_QColorDialog : public QColorDialog {
class OnX_QColorDialog : public QWidget {
  Q_OBJECT
public:
#if QT_VERSION < 0x040000
  virtual void show();
  virtual void hide();
#else
  void map();
  void unmap();
#endif
public:
  OnX_QColorDialog(const char*);
  virtual ~OnX_QColorDialog();
  std::string value() const;
signals:
  void ok();
private:
  std::string fValue;
};

// Have a QWidget to handle the "save" case.
// Used only with Qt4.
//class OnX_QFileDialog : public QWidget {
class OnX_QFileDialog : public QDialog {
  Q_OBJECT
public:
  void map();
  void unmap();
public:
  OnX_QFileDialog(const char*);
  virtual ~OnX_QFileDialog();
  std::string value() const;
  enum AcceptMode { AcceptOpen, AcceptSave };
  void setAcceptMode(AcceptMode);
  void setFilter(const std::string&);
  void setDirectory(const std::string&);
signals:
  void ok();
private:
  AcceptMode fMode;
  std::string fFilter;
  std::string fDirectory;
  std::string fValue;
};

class OnX_QMenu : public QWidget {
  Q_OBJECT
public:
  OnX_QMenu(const char*,QPopupMenu&);
  virtual ~OnX_QMenu();
  QPopupMenu& popupMenu() const;
private:
  QPopupMenu& fPopup;
};

class OnX_QMenuItem : public QWidget {
  Q_OBJECT
public:
  OnX_QMenuItem(const char*,QPopupMenu&,int = 0);
  virtual ~OnX_QMenuItem();
  QPopupMenu& popupMenu() const;
private:
  QPopupMenu& fPopup;
  int fID;
};

class OnX_QToggleMenuItem : public OnX_QMenuItem {
  Q_OBJECT
public:
  OnX_QToggleMenuItem(const char*,QPopupMenu&,int = 0);
  virtual ~OnX_QToggleMenuItem();
};

class QtCallback : public QObject {
  Q_OBJECT
public:
  QtCallback(OnX::Action&);
  virtual ~QtCallback();
  OnX::Action& action() const;
public slots:
  void destroy();
  void execute();
  void execute2(QCallbackData&);
  //#if QT_VERSION < 0x040000
  //  void execute3(QListViewItem*);
  //#else
  void execute3(Q3ListViewItem*);
  //#endif
private:
  OnX::Action& fAction;
};

#if QT_VERSION < 0x040000
namespace OnX {
class QtMessage {
public:
  QtMessage(QtUI&,Slash::Core::IMessage*);
  ~QtMessage();
public:
  QtUI& fUI;
  Slash::Core::IMessage* fMessage;
};
}
class OnX_QMessageWidget : public QWidget {
protected:
  void customEvent(QCustomEvent*);
};
#else
namespace OnX {
class QCustomEvent : public QEvent {
public:
  QCustomEvent(QtUI&,Slash::Core::IMessage*);
  virtual ~QCustomEvent();
public:
  QtUI& fUI;
  Slash::Core::IMessage* fMessage;
};
}
class OnX_QMessageWidget : public QWidget {
protected:
  void customEvent(QEvent*);
};
#endif

#if QT_VERSION < 0x040000
#else
#undef QPopupMenu
#endif

#endif
